#property copyright ""
#property link      ""

#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1  clrGreen      //uptrend
#property indicator_width1  2  
#property indicator_color2  clrIndianRed  //downtrend
#property indicator_width2  2  
#property indicator_color3  clrDarkGreen // First Stochastic line
#property indicator_width3  2  
#property strict

extern ENUM_TIMEFRAMES TimeFrame        = PERIOD_CURRENT;  // Time frame to use
extern int             StochPeriod      = 14;              // Stochastic period
extern int             SlowingPeriod    = 3;               // Stochastic slowing (smoothing) period
extern ENUM_STO_PRICE  StochPrice       = 0;               // Price to use
extern bool            ShowArrows       = false;           // Display arrows on chart?
extern string          arrowsIdentifier = "sto Arrows";    // Arrows unique ID
extern double          arrowsUpperGap   = 1.0;             // Gap from price for up arrows
extern double          arrowsLowerGap   = 1.0;             // Gap from price for down arrow
extern color           arrowsUpColor    = clrLimeGreen;    // Color for up arrow
extern color           arrowsDnColor    = clrRed;          // Color for down arrow
extern int             arrowsUpCode     = 241;             // Code for up arrow
extern int             arrowsDnCode     = 242;             // Code for down arrow
extern bool            ShowVLines       = false;           // Display vertical lines on chart?
extern string          linesIdentifier  = "sto VLines";    // Vertical lines unique ID
extern color           linesUpColor     = clrLimeGreen;    // Color for up line
extern color           linesDnColor     = clrRed;          // Color for down line
extern ENUM_LINE_STYLE linesStyle       = STYLE_SOLID;     // Style for lines
extern int             linesWidth       = 2;               // Width for lines
extern bool            linesBckg        = false;           // Lines in background?
extern bool            alertsOn         = true;            // Turn alerts on?
extern bool            alertsOnCurrent  = false;           // Alerts on current (still opened) bar
extern bool            alertsMessage    = true;            // Alerts should show pop-up message
extern bool            alertsSound      = false;           // Alerts should play alert sound
extern bool            alertsPushNotif  = false;           // Alerts should send push notification
extern bool            alertsEmail      = false;           // Alerts should send email

double upB[];
double loB[];
double sto[];
double trend[];
//+------------------------------------------------------------------+
int init() 
{
   IndicatorBuffers(4);
   SetIndexBuffer(0,upB); SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,loB); SetIndexStyle(1,DRAW_HISTOGRAM); 
   SetIndexBuffer(2,sto);
   SetIndexBuffer(3,trend);
   IndicatorShortName(timeFrameToString(TimeFrame)+" Stoch Histogram ("+(string)StochPeriod+","+(string)SlowingPeriod+")");
return(0);
}
int deinit()
{
   string lookFor       = arrowsIdentifier+":";
   string lookForToo    = linesIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   int    lookForTooLength = StringLen(lookForToo);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
         if (StringSubstr(objectName,0,lookForTooLength) == lookForToo) ObjectDelete(objectName);
   }
   return(0);
}

//
//
//
//
//

int start() 
{
int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
            int limit=MathMin(MathMax(Bars-counted_bars,3*TimeFrame/Period()),Bars-1);
            
   //
   //
   //
   //
   //
   
   for(int i=limit; i>=0; i--) 
   {
      int y  = iBarShift(NULL,TimeFrame,Time[i]);
      sto[i] = iStochastic(NULL,TimeFrame,StochPeriod,1,SlowingPeriod,0,StochPrice,MODE_MAIN,y)-50;      
      
      //
      //
      //
      //
      //
      
      upB[i]   = EMPTY_VALUE;
      loB[i]   = EMPTY_VALUE;
      if (i<Bars-1) trend[i] = trend[i+1];
      if(sto[i]>0)  trend[i] = 1;
      if(sto[i]<0)  trend[i] =-1;
      if (trend[i] == 1) upB[i] = sto[i];
      if (trend[i] ==-1) loB[i] = sto[i];   
         
      //
      //
      //
      //
      //
     
      if (ShowArrows)
      {
         ObjectDelete(arrowsIdentifier+":"+(string)Time[i]);
         if (i<Bars-1 && trend[i]!=trend[i+1])
         {
            if (trend[i] == 1)  drawArrow(i,arrowsUpColor,arrowsUpCode,false);
            if (trend[i] ==-1)  drawArrow(i,arrowsDnColor,arrowsDnCode, true);
          }
       }
     
      if (ShowVLines)
      {
         ObjectDelete(linesIdentifier+":"+(string)Time[i]);
         if (i<Bars-1 && trend[i]!=trend[i+1])
         {
            if (trend[i] == 1)  drawVLine(i,linesUpColor,linesStyle,linesWidth,linesBckg);
            if (trend[i] ==-1)  drawVLine(i,linesDnColor,linesStyle,linesWidth,linesBckg);
          }
       }
   }
   
   //
   //
   //
   //
   //
   
   if (alertsOn)
   {
      int whichBar = 1; if (alertsOnCurrent) whichBar = 0; whichBar = iBarShift(NULL,0,iTime(NULL,TimeFrame,whichBar));
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] ==  1) doAlert(whichBar,"up");
         if (trend[whichBar] == -1) doAlert(whichBar,"down");
      }
    }
return(0);
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//
//-------------------------------------------------------------------
void drawArrow(int i,color theColor,int theCode,bool up)
{
   string name = arrowsIdentifier+":"+(string)Time[i];
   double gap  = iATR(NULL,0,20,i);   
   
      //
      //
      //
      //
      //
      
      ObjectCreate(name,OBJ_ARROW,0,Time[i],0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         if (up)
               ObjectSet(name,OBJPROP_PRICE1,High[i] + arrowsUpperGap * gap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i]  - arrowsLowerGap * gap);
}

//
//
//
//
//
//-------------------------------------------------------------------
void drawVLine(int i,color c,int s,int w, bool bck)
{
   string name = linesIdentifier+":"+(string)Time[i];  
      
         ObjectCreate(name,OBJ_VLINE,0,Time[i],0);
         ObjectSet(name,OBJPROP_COLOR,c);
         ObjectSet(name,OBJPROP_STYLE,s);
         ObjectSet(name,OBJPROP_WIDTH,w);
         ObjectSet(name,OBJPROP_BACK,bck);
         ObjectSet(name,OBJPROP_SELECTABLE,false);
}

//
//
//
//
//
//-------------------------------------------------------------------
void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," ",timeFrameToString(_Period)," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," StochHistogram trend changed to ",doWhat);
          if (alertsMessage)   Alert(message);
          if (alertsEmail)     SendMail(Symbol()+" StochHistogram",message);
          if (alertsPushNotif) SendNotification(message);
          if (alertsSound)     PlaySound("alert2.wav");
   }
}